use crate::{
    build_tools::{cmake::Cmake, git::Git},
    profile::Profile,
    termcolor::TermColor,
    ui::job_worker::job::{FuncWorkerData, FuncWorkerOut, WorkerJob},
    util::file_utils::{copy_file, rm_rf},
};
use std::{
    collections::{HashMap, VecDeque},
    fs::create_dir_all,
    path::Path,
};

pub fn get_build_vapor_jobs(profile: &Profile, clean_build: bool) -> VecDeque<WorkerJob> {
    let mut jobs = VecDeque::<WorkerJob>::new();
    jobs.push_back(WorkerJob::new_printer(
        "Building VapoR...",
        Some(TermColor::Blue),
    ));

    let git = Git {
        repo: profile
            .ovr_comp
            .repo
            .as_ref()
            .unwrap_or(&"https://github.com/micheal65536/VapoR.git".into())
            .clone(),
        dir: profile.ovr_comp.path.clone(),
        branch: profile
            .ovr_comp
            .branch
            .as_ref()
            .unwrap_or(&"master".into())
            .clone(),
    };

    jobs.extend(git.get_pre_build_jobs(profile.pull_on_build));

    let build_dir = profile.ovr_comp.path.join("build");
    let cmake = Cmake {
        env: None,
        vars: Some({
            let mut cmake_vars: HashMap<String, String> = HashMap::new();
            for (k, v) in [
                ("VAPOR_LOG_SILENT=ON", "ON"),
                ("CMAKE_BUILD_TYPE", "RelWithDebInfo"),
            ] {
                cmake_vars.insert(k.to_string(), v.to_string());
            }
            cmake_vars
        }),
        source_dir: profile.ovr_comp.path.clone(),
        build_dir: build_dir.clone(),
    };
    if !Path::new(&build_dir).is_dir() || clean_build {
        rm_rf(&build_dir);
        jobs.push_back(cmake.get_prepare_job());
    }
    jobs.push_back(cmake.get_build_job());
    jobs.push_back(WorkerJob::Func(FuncWorkerData {
        func: Box::new(move || {
            let dest_dir = build_dir.join("bin/linux64");
            if let Err(e) = create_dir_all(&dest_dir) {
                return FuncWorkerOut {
                    success: false,
                    out: vec![format!(
                        "failed to create dir {}: {e}",
                        dest_dir.to_string_lossy()
                    )],
                };
            }
            copy_file(
                &build_dir.join("src/vrclient.so"),
                &dest_dir.join("vrclient.so"),
            );

            FuncWorkerOut {
                success: true,
                out: Vec::default(),
            }
        }),
    }));

    jobs
}
